@@ -50,14 +50,14 @@ def open(path):
 	'''Open the profile browser with the specified JSON file'''
 	path = pathlib.Path(path)
 	if not path.exists() or not path.is_file():
-		raise click.ClickException('The profile file is invalid or does not exist')
+		raise util.SailException('The profile file is invalid or does not exist')
 
 	with path.open('r') as f:
 		try:
 			profile_data = json.load(f)
 			xhprof_data = profile_data['xhprof']
 		except:
-			raise click.ClickException('This profile file is invalid, invalid or incomplete JSON')
+			raise util.SailException('This profile file is invalid, invalid or incomplete JSON')
 
 	del profile_data['xhprof']
 
@@ -134,7 +134,7 @@ def run(ctx, url):
 	config = util.config()
 
 	if 'profile_key' not in config:
-		raise click.ClickException('Profile key not found in .sail/config.json')
+		raise util.SailException('Profile key not found in .sail/config.json')
 
 	url = urlparse(url)
 	host = config['hostname']
@@ -144,10 +144,10 @@ def run(ctx, url):
 	query = '?' + query
 	url_path = url.path if url.path else '/'
 
-	click.echo('# Profiling')
-	click.echo('- Server: %s' % host)
-	click.echo('- Host: %s' % url.netloc)
-	click.echo('- Request: GET %s%s' % (url_path, query))
+	util.heading('Profiling')
+	util.item('Server: %s' % host)
+	util.item('Host: %s' % url.netloc)
+	util.item('Request: GET %s%s' % (url_path, query))
 
 	headers = {
 		'Host': url.netloc,
@@ -159,12 +159,12 @@ def run(ctx, url):
 	session = requests.Session()
 
 	try:
-		response = session.send(request)
+		response = session.send(request, allow_redirects=False)
 	except:
-		raise click.ClickException('Could not make profiling request.')
+		raise util.SailException('Could not make profiling request.')
 
 	if 'X-Sail-Profile' not in response.headers:
-		raise click.ClickException('X-Sail-Profile header not found in response. Check your profile key.')
+		raise util.SailException('X-Sail-Profile header not found in response. Check your profile key.')
 
 	filename = response.headers['X-Sail-Profile']
 	profile_path = ctx.invoke(download, path=filename)
@@ -178,7 +178,7 @@ def key(header):
 	config = util.config()
 
 	if 'profile_key' not in config:
-		raise click.ClickException('Profile key not found in .sail/config.json')
+		raise util.SailException('Profile key not found in .sail/config.json')
 
 	if header:
 		click.echo('X-Sail-Profile: %s' % config['profile_key'], nl=False)
@@ -195,11 +195,12 @@ def curl(ctx, command):
 	config = util.config()
 
 	if 'profile_key' not in config:
-		raise click.ClickException('Profile key not found in .sail/config.json')
+		raise util.SailException('Profile key not found in .sail/config.json')
 
 	command = list(command)
 	command = ['-s', '-v', '-H', 'X-Sail-Profile: %s' % config['profile_key']] + command
-	click.echo('# Running cURL with profiling headers', err=True)
+
+	util.heading('Running cURL with profiling headers', err=True)
 
 	# TODO: Maybe add the SAIL_NOCACHE query var
 	p = subprocess.Popen(['curl'] + command, stdout=subprocess.PIPE,
@@ -208,11 +209,11 @@ def curl(ctx, command):
 	stdout, stderr = p.communicate()
 
 	if p.returncode != 0:
-		raise click.ClickException('An error occurred in cURL. Please try again.')
+		raise util.SailException('An error occurred in cURL. Please try again.')
 
 	matches = re.search(r'^<\s*X-Sail-Profile: (.+)$', stderr, re.MULTILINE)
 	if not matches:
-		raise click.ClickException('Could not find profile filename from headers')
+		raise util.SailException('Could not find profile filename from headers')
 
 	filename = matches.group(1)
 	profile_path = ctx.invoke(download, path=filename)
@@ -220,19 +221,24 @@ def curl(ctx, command):
 
 @profile.command()
 @click.argument('path', nargs=1)
-def download(path):
+@click.pass_context
+def download(ctx, path):
 	'''Download a profile JSON from the production server'''
 	root = util.find_root()
 	config = util.config()
 
+	# Invoked by user, not forwarder/invoked by
+	if ctx.parent and ctx.parent.command == profile:
+		util.heading('Downloading profile')
+
 	if 'profile_key' not in config:
-		raise click.ClickException('Profile key not found in .sail/config.json')
+		raise util.SailException('Profile key not found in .sail/config.json')
 
 	profiles_dir = pathlib.Path(root + '/.profiles')
 	profiles_dir.mkdir(parents=True, exist_ok=True)
 
 	dest_filename = datetime.now().strftime('%Y-%m-%d-%H%M%S.xhprof.json')
-	click.echo('- Downloading profile from %s' % path)
+	util.item('Downloading profile from %s' % path)
 
 	args = ['-t']
 	source = 'root@%s:%s' % (config['hostname'], path)
@@ -240,9 +246,9 @@ def download(path):
 	returncode, stdout, stderr = util.rsync(args, source, destination, default_filters=False)
 
 	if returncode != 0:
-		raise click.ClickException('An error occurred in rsync. Please try again.')
+		raise util.SailException('An error occurred in rsync. Please try again.')
 
-	click.echo('- Cleaning up production')
+	util.item('Cleaning up production')
 
 	p = subprocess.Popen(['ssh',
 		'-i', '%s/.sail/ssh.key' % root,
@@ -257,9 +263,10 @@ def download(path):
 		util.loader()
 
 	if p.returncode != 0:
-		raise click.ClickException('An error occurred in SSH. Please try again.')
+		raise util.SailException('An error occurred in SSH. Please try again.')
 
-	click.echo('- Profile saved to .profiles/%s' % dest_filename)
+	util.item('Profile saved to .profiles/%s' % dest_filename)
+	click.echo()
 	return profiles_dir / dest_filename
 
 @profile.command()
@@ -269,12 +276,12 @@ def clean():
 	config = util.config()
 
 	# Delete local profiles
-	click.echo('# Cleaning up')
-	click.echo('- Deleting local profiles')
+	util.heading('Cleaning up')
+	util.item('Deleting local profiles')
 	profiles_dir = pathlib.Path(root + '/.profiles')
 	shutil.rmtree(profiles_dir)
 
-	click.echo('- Deleting production profiles')
+	util.item('Deleting production profiles')
 	p = subprocess.Popen(['ssh',
 		'-i', '%s/.sail/ssh.key' % root,
 		'-o', 'UserKnownHostsFile="%s/.sail/known_hosts"' % root,
@@ -285,12 +292,12 @@ def clean():
 	])
 
 	while p.poll() is None:
-		util.loader()
+		pass
 
 	if p.returncode != 0:
-		raise click.ClickException('An error occurred in SSH. Please try again.')
+		raise util.SailException('An error occurred in SSH. Please try again.')
 
-	click.echo('- Done')
+	util.success('Cleanup complete')
 
 def _render_summary(pad, totals):
 	run_id = datetime.fromtimestamp(totals['timestamp']).strftime('%Y-%m-%d %H:%M:%S')
